const path = require('path');
const fs = require('fs');
const HtmlWebpackPlugin = require('html-webpack-plugin');


module.exports = {
  devtool: 'inline-source-map',
  mode: 'development',
  entry: () => {
    result = fs.readdirSync(__dirname + '/demos').reduce(function (entries, dir) {
      if(dir === 'resources')
        return entries
      if (fs.statSync(path.join(__dirname + '/demos', dir)).isDirectory() && dir !== '__build__')
      {
        let entryFileTs = path.join(__dirname + '/demos', dir, 'index.ts')
        let entryFileTsx = path.join(__dirname + '/demos', dir, 'index.tsx')
        if(fs.existsSync(entryFileTs))
          entries[dir] = entryFileTs
        else if(fs.existsSync(entryFileTsx))
          entries[dir] = entryFileTsx
      }

      return entries
    }, {})
    return result
  },
  experiments: {
    topLevelAwait: true
  },
  devServer: {
    //contentBase: path.join(__dirname, 'demos'),
    static: {
      directory: path.join(__dirname, 'demos')
    },
    compress: true,
    port: 9001,
    open: true,
    host: 'localhost',
    /*devMiddleware: {writeToDisk: true}*/
  },
  module: {
    rules: [
      {
        test: /.(ts|tsx)$/,
        use: {
          loader : 'babel-loader',
          options: {
          	presets: [
              '@babel/preset-typescript',
              '@babel/preset-env',
              '@babel/preset-react'
            ],
          }
        }
      },
      {
        test: /\.css$/,
        use: [
          'style-loader',
          {
            loader: 'css-loader',
            options: {
              importLoaders: 1,
              modules: true
            }
          }
        ],
        include: /\.module\.css$/
      },
      {
        test : /\.css$/,
        use: ['style-loader', 'css-loader'],
        exclude: /\.module\.css$/
      },
      {
        test : /\.scss$/,
        use: [
          {
            loader: 'style-loader'
          },
          {
            loader: 'css-loader'
          },
          {
            loader: 'sass-loader'
          }
        ]
      },
      {
        test: /\.(jpg|png|svg)$/,
        loader: 'url-loader',
        options: {
          limit: 25000,
        },
      },
      {
        test: /\.txt/,
        type: 'asset/source'
      }
    ]
  },
  resolve: {
    alias: {
      'tuo-document-react': path.join(__dirname, '..','src/index.tsx')
    },
    extensions: ['.ts', '.js', '.tsx']
  },
  plugins: (() => {
    let result = []
    fs.readdirSync(__dirname + '/demos').forEach(function(dir){
      if(dir === 'resources')
        return
      if(fs.statSync(path.join(__dirname + '/demos', dir)).isDirectory()){
      result.push( new HtmlWebpackPlugin({
          template: path.join(__dirname + '/demos', dir, 'index.html'),
          filename: `${dir}/index.html`,
          chunks: [dir]
        }))
      }
    })

    return result
  })()
}; 
