import React, {useState, useEffect} from 'react'
import styles from './styles/viewToolBar.module.css'
import focusAutoSvg from './imgs/Focus-Auto.svg';
import focusAllSvg from './imgs/FocusAll-Auto.svg';
import visibleOnlySvg from './imgs/showSelected.svg';
import showAllSvg from './imgs/showAll.svg';
import visibleSvg from './imgs/visible.svg';
import invisibleSvg from './imgs/invisible.svg';
import wireframeSvg from './imgs/wireframe.svg';
import measureSvg from './imgs/measure.svg';
import opacitySvg from './imgs/opacity.svg';
import inOpacitySvg from './imgs/inopacity.svg';
import asisSvg from './imgs/asis.svg';
import helpSvg from './imgs/help.svg'
import { utils as coreUtils, types } from 'tuo-document';
import Markdown from 'react-markdown'
import remarkGfm from 'remark-gfm'
import {manualText} from './manual'
import Hotkeys from 'react-hot-keys';


type ViewToolBarProps = {
  doc: any,
  hasFocus: boolean
}


const operations = [
  {
    isSpacer: true
  },
  {
    isDivider: true
  },
  {
    img: focusAllSvg,
    title: '聚焦全部对象',
    action: 'focusAll'
  },
  {
    img: focusAutoSvg,
    title: '聚焦选中对象',
    action: 'focusTarget'
  },
  {
    img: showAllSvg,
    title: '显示所有对象',
    action: 'showAll'
  },
  {
    img: visibleSvg,
    title: '显示被选中对象',
    action: 'showSelected'
  },
  {
    img: invisibleSvg,
    title: '隐藏选中对象',
    action: 'hideSelected'
  },
  {
    img: visibleOnlySvg,
    title: '仅显示选中对象',
    action: 'visibleOnly'
  },
  {
    isDivider: true
  },
  {
    img: asisSvg,
    title: '实体渲染',
    action: 'asis'
  },
  {
    img: wireframeSvg,
    title: '线框渲染',
    action: 'wireframe'
  },
  {
    img: opacitySvg,
    title: '设选中对象为透明',
    action: 'opacity'
  },
  {
    img: inOpacitySvg,
    title: '设选中对象为非透明',
    action: 'inOpacity'
  },
  {
    isDivider: true
  },
  {
    img: measureSvg,
    title: '测量模式',
    action: 'measure'
  },
  {
    isDivider: true
  },
  {
    img: helpSvg,
    title: '帮助',
    action: 'help'
  }
] 



export default function ViewToolBar({doc, hasFocus}){
  const [measureDialog, setMeasureDialog] = useState(false)
  const [manualDialog, setManualDialog] = useState(false)
  

  if(!doc)
    return null

  const getSelectedId = () => {
    const raycast = doc.mainStore.getState().raycast

    if (raycast.selected.length == 0) return undefined;
    let firstSelected = raycast.selected[0];
    if (typeof firstSelected === 'string') return firstSelected;
    return coreUtils.getNodeKey(firstSelected.object);
  }

  const getSelectedIds = () => {
    const raycast = doc.mainStore.getState().raycast
    return raycast.selected.map((selected) => {
      return typeof selected === 'string'
        ? selected
        : coreUtils.getNodeKey(selected.object);
    });
  }

  const getAllNodesVisiblePayload = () => {
    const nodes = doc.mainStore.getState().nodes.present
    return nodes.allIds.reduce((previous, currentValue) => {

      previous[currentValue] = true;
      return previous;
    }, {});
  }

  const checkIsGroupNode = (id) => {
    const nodes = doc.mainStore.getState().nodes.present
    return nodes.byId[id].type === 'group';
  }

  const getAllNodesVisibleExceptPayload = () => {
    const nodes = doc.mainStore.getState().nodes.present
    let selectedIds = getSelectedIds().map((id) =>
      coreUtils.getSelectedObjectId(doc, id),
    );
    const tartgetIds = nodes.allIds.reduce((previous, currentValue) => {

      if (!nodes.byId[currentValue].options)
        return previous

      if (selectedIds.findIndex(v => v === currentValue) > -1) {
        previous[currentValue] = true;
      } else if (nodes.byId[currentValue].options.type === 'Part' && nodes.byId[currentValue].options.tuzhiAdded) {
        previous[currentValue] = false;
      } else if (
        checkIsGroupNode(currentValue) ||
        nodes.byId[currentValue].options.type !== 'Object'
      ) {
        previous[currentValue] = true;
      } else previous[currentValue] = false;
      return previous;
    }, {});

    // console.log('tartgetIds:', tartgetIds)
    return tartgetIds
  }

  const getAllSelectedVisiblePayload = (value) => {
    return getSelectedIds()
      .map((id) => coreUtils.getSelectedObjectId(doc, id))
      .reduce((previous, currentValue) => {
        previous[currentValue] = value;
        return previous;
      }, {});
  }

  const getAllSelectedRenderTypePayload = (value) => {
    return getSelectedIds()
      .map((id) => coreUtils.getSelectedObjectId(doc, id))
      .reduce((previous, currentValue) => {
        previous[currentValue] = value;
        return previous;
      }, {});
  }

  const onClick = (action) => {
    switch (action) {
      case 'focusAll':
        doc.viewDocument.fitScene(true, true);
        break;
      case 'showAll':
        //this.props.setNodesVisible(this.getAllNodesVisiblePayload())
        doc.dispatch({
          type: types.SET_NODES_VISIBLE,
          payload: getAllNodesVisiblePayload(),
        });
        break;
      case 'focusTarget':
        //this.props.fitSelected(this.getSelectedId())
        doc.viewDocument.fitObject(getSelectedId());
        break;
      case 'visibleOnly':
        //this.props.setNodesVisible(getAllNodesVisibleExceptPayload())
        doc.dispatch({
          type: types.SET_NODES_VISIBLE,
          payload: getAllNodesVisibleExceptPayload(),
        });
        break;
      case 'showSelected':
        //this.props.setNodesVisible(this.getAllSelectedVisiblePayload(true))
        doc.dispatch({
          type: types.SET_NODES_VISIBLE,
          payload: getAllSelectedVisiblePayload(true),
        });
        break;
      case 'hideSelected':
        //this.props.setNodesVisible(this.getAllSelectedVisiblePayload(false))
        doc.dispatch({
          type: types.SET_NODES_VISIBLE,
          payload: getAllSelectedVisiblePayload(false),
        });
        break;
      case 'asis':
        //this.props.setNodesRenderType(this.getAllSelectedRenderTypePayload('asis'))
        doc.dispatch({
          type: types.SET_NODES_RENDERTYPE,
          payload: getAllSelectedRenderTypePayload('asis'),
        });
        break;
      case 'wireframe':
        //this.props.setNodesRenderType(this.getAllSelectedRenderTypePayload('wireframe'))
        doc.dispatch({
          type: types.SET_NODES_RENDERTYPE,
          payload: getAllSelectedRenderTypePayload('wireframe'),
        });
        break;
      case 'opacity':
        doc.dispatch({
          type: types.SET_NODES_MATERIAL,
          payload: getSelectedIds()
          .map((id) => coreUtils.getSelectedObjectId(doc, id))
          .reduce((acc, id) => {
            acc[id] = {face: {transparent: true, opacity: 0.5}}
            return acc
          }, {})
        })
        break;
      case 'inOpacity':
        doc.dispatch({
          type: types.SET_NODES_MATERIAL,
          payload: getSelectedIds()
          .map((id) => coreUtils.getSelectedObjectId(doc, id))
          .reduce((acc, id) => {
            acc[id] = {face: {transparent: false, opacity: 1}}
            return acc
          }, {})
        })
        break
      case 'measure':
        setMeasureDialog(!measureDialog)
        break;
      case 'help':
        setManualDialog(!manualDialog)
        break;
        /*
      case 'measure':
        onMenuClick()
        break;
      case 'opacity':
        setDialog(actionType);
        break;*/
    }
  }

  const onKeyDown = (keyName, e, handle) => {
    switch (keyName) {
      case 'h':
        onClick('help')
        break
      case 'shift+f':
        onClick('focusAll')
        break
      case 'f':
        onClick('focusTarget')
        break
      case 'shift+v':
        onClick('showAll')
        break
      case 'v':
        onClick('showSelected')
        break
      case 'c':
        onClick('hideSelected')
        break
      case 'z':
        onClick('visibleOnly')
        break
      case 'a':
        onClick('asis')
        break
      case 'w':
        onClick('wireframe')
        break
      case 't':
        onClick('opacity')
        break
      case 'shift+t':
        onClick('inOpacity')
        break
      case 'm':
        onClick('measure')
        break
    }
  }


  return <div
    className = {styles.container}
  >
    {operations.map((operation, index) => {
      if(operation.isSpacer)
        return <div key = {index} className = {styles.spacer}></div>
      if(operation.isDivider)
        return <div key = {index} className = {styles.divider}></div>

      return <ToolBarIcon
        img = {operation.img}
        title = {operation.title}
        action = {operation.action}
        onClick = {onClick}
        key = {index}
      />

    })}
    {measureDialog ? <MeasureDialog doc = {doc} onClose = {() => setMeasureDialog(false)}/> : null}
    {manualDialog ? <ManualDialog onClose = {() => setManualDialog(false)}/> : null}
    {hasFocus ? <Hotkeys
      keyName='h,shift+f,f,shift+v,v,z,a,w,t,shift+t,m,c'
      onKeyDown={onKeyDown}
    /> : null}
  </div>
} 


function ToolBarIcon({img, title, action, onClick}){
  const [showTitle, setShowTitle] = useState(false)

  const onMouseOut = () => {
    setShowTitle(false)
  }

  const onMouseOver = () => {
    setShowTitle(true)
  }

  return <div className={styles.iconContainer}>
      <img
        className={styles.iconImage}
        width={28}
        height={28}
        onMouseOut={onMouseOut}
        onMouseOver={onMouseOver}
        src={img}
        onClick={() => onClick(action)}
      />
      {showTitle ? (
        <div className={styles.iconTitle}>{title}</div>
      ) : null}
  </div>
}


function MeasureDialog({doc, onClose}){

  useEffect(() => {
    doc.dispatch({
      type: types.START_PROCESS,
      payload: {
        type: 'analysis',
        config: {}
      }
    })
    return () => {
      doc.dispatch({
        type: types.QUIT_PROCESS,
      })

    }
  }, [])


  return <div className = {styles.measureContainer}>
    <div style = {{
      width: '100px'
    }}>测量模式</div>
    <button 
      className = {styles.closeButton}
      onClick = {() => {onClose()}}>x</button>
  </div>
}


function ManualDialog({onClose}){
  return <div className = {styles.manualContainer}>
    <div
      style = {{
        borderBottom: '1px solid #e0e0e0',
        padding: '4px'
      }}
    >
      <button 
        className = {styles.closeButton}
        onClick = {() => {onClose()}}>x</button>
    </div>
    <div
      style = {{
        padding: '4px'
      }}
    >
      <Markdown remarkPlugins={[remarkGfm]}>{manualText}</Markdown>
    </div>
  </div>
}
